#!/usr/bin/python

import os
import sys
import logging

import ivle.conf
import ivle.db
import ivle.chat
import ivle.makeuser

# usage:
#   usrmgt-server <port> <magic>

# User management operations:
#   - Create local user
#   - [Re]Create jail for a user
#       - Create a svn repository for a user
#           - create repository
#           - svn config
#           - svn auth
#       - /etc/passwd entry
#   - Disable a user's account
#   - Enable a user's account
#   - Remove a user
#   - Rebuild svn config
#   - Rebuild svn auth file
#   - Rebuild passwd + push to nodes.

def activate_user(props):
    """Create the on-disk stuff for the given user.
       Sets the state of the user in the db from pending to enabled.
       Expected properties:
        login       - the user name for the jail
                      STRING REQUIRED
       Return Value: None
    """

    os.umask(0022) # Bad, but start_server sets it worse.

    login = props['login']

    db = ivle.db.DB()

    try:

        # FIXME: check we're pending

        details = db.get_user(login)

        # make svn config/auth

        repopath = os.path.join(ivle.conf.svn_repo_path, 'users', login)
        logging.debug("Creating user's Subversion repository")
        ivle.makeuser.make_svn_repo(repopath, throw_on_error=True)

        rebuild_svn_config(props)

        logging.debug("Adding Subversion authentication")
        passwd = ivle.makeuser.make_svn_auth(login, throw_on_error=True)
        logging.debug("passwd: %s" % passwd)

        logging.debug("Creating jail")
        ivle.makeuser.make_jail(login, details.unixid, svn_pass=passwd)

        logging.info("Enabling user")
        db.update_user(login, state='enabled')

        return {"response": "okay"}

    finally:
        db.close()

def rebuild_svn_config(props):
    """Rebuilds the svn config file
    Return value:
        response (okay, failure)
    """
    try:
        ivle.makeuser.rebuild_svn_config()
    except Exception, e:
        logging.warning('Rebuild of Subversion authorization config failed!')
        return{'response': 'failure', 'msg': repr(e)}

    return {'response': 'okay'}

def rebuild_svn_group_config(props):
    """Rebuilds the svn group config file
    Return value:
        response (okay, failure)
    """
    try:
        ivle.makeuser.rebuild_svn_group_config()
    except Exception, e:
        logging.warning(
            'Rebuild of Subversion group authorization config failed!')
        return{'response': 'failure', 'msg': repr(e)}

    return {'response': 'okay'}

def create_group_repository(props):
    """Creates on disk repository for the given group
    Expected properties:
        subj_short_name, year, semester, groupnm
    Return value:
        response (okay, failure)
    """

    subj_short_name = props['subj_short_name']
    year = props['year']
    semester = props['semester']
    groupnm = props['groupnm']

    namespace = "_".join([subj_short_name, year, semester, groupnm])
    repopath = os.path.join(ivle.conf.svn_repo_path, 'groups', namespace)
    logging.debug("Creating Subversion repository %s"%repopath)
    try:
        ivle.makeuser.make_svn_repo(repopath)
    except Exception, e:
        logging.error("Failed to create Subversion repository %s: %s"%
            (repopath,repr(e)))
        return {'response': 'failure', 'msg': repr(e)}

    return {'response': 'okay'}

actions = {
        'activate_user':activate_user,
        'create_group_repository':create_group_repository,
        'rebuild_svn_config':rebuild_svn_config,
        'rebuild_svn_group_config':rebuild_svn_group_config,
    }

def initializer():
    try:
        pidfile = open('/var/run/usrmgt-server.pid', 'w')
        pidfile.write('%d\n' % os.getpid())
        pidfile.close()
    except IOError, (errno, strerror):
        print "Couldn't write PID file. IO error(%s): %s" % (errno, strerror)
        sys.exit(1)

def dispatch(props):
    logging.debug(repr(props))
    action = props.keys()[0]
    return actions[action](props[action])

if __name__ == "__main__":
    pid = os.getpid()

    logging.basicConfig(filename="/var/log/usrmgt.log", level=logging.INFO)
    logging.info("Starting usrmgt server on port %d (pid = %d)" %
                 (ivle.conf.usrmgt_port, pid))

    ivle.chat.start_server(ivle.conf.usrmgt_port, ivle.conf.usrmgt_magic,
                           True, dispatch, initializer)
